/*
 * Copyright (C) 2012-2013, Eric Wong <normalperson@yhbt.net>
 * License: GPLv3 or later (see COPYING for details)
 */
#ifdef HAVE_OPEN_MEMSTREAM
# define my_memstream_close(fp,dst,bytes) fclose((fp))
#else
static FILE * my_open_memstream(char **ptr, size_t *sizeloc)
{
	FILE *fp;

	do {
		errno = 0;
		fp = tmpfile();
		if (fp != NULL)
			return fp;
	} while (errno == EINTR);

	syslog(LOG_ERR, "tmpfile(open_memstream fallback) failed: %m");
	/* we'd hit mog_oom() anyways */
	abort();
}

#define open_memstream(ptr,sizeloc) my_open_memstream((ptr),(sizeloc))

static int my_memstream_close(FILE *fp, char **dst, size_t *bytes)
{
	long pos;

	errno = 0;
	pos = ftell(fp);

	if (pos >= 0) {
		rewind(fp);
		*bytes = (size_t)pos;
		*dst = xmalloc(*bytes);
		if (fread(*dst, 1, *bytes, fp) == *bytes)
			return fclose(fp);
	} else {
		assert(errno != EBADF
		       && "EBADF in stdio/open_memstream replacement");
	}

	*bytes = 0;
	syslog(LOG_ERR, "stdio/open_memstream replacement failed: %m");

	/*
	 * if ftell() fails, fclose() may fail due to an I/O error,
	 * too, but at least hope we release memory..
	 */
	(void)fclose(fp);

	return 0;
}
#endif /* !HAVE_OPEN_MEMSTREAM */
